/*******************************************************************************
 * C H A M E L E O N    S. D. K.                                               *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/dspthru/main.c                      $
 *     $Date:: 9/12/02 20:32                                                   $
 * $Revision:: 5                                                               $
 *-----------------------------------------------------------------------------*
 * This file is part of the Chameleon Software Development Kit                 *
 *                                                                             *
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

/******************************************************************************
 This example shows the fundamentals of handling audio I/O in the Chameleon, 
 in several ways: polling, interrupt and DMA. To select one of these available
 modes, you have to define one of the symbols POLLING or INTERRUPT to use the 
 polling or interrupt method, or if any of them are not defined, to use the DMA.

 It only uses the volume potentiometer to adjust the desired volume
 using a pre-filled table with a variable gain from -90 dB to 0 dB.
 ******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <rtems.h>
#include <chameleon.h>

//#define POLLING
//#define INTERRUPT

#if defined(POLLING)
  #include "dsp/dsp_polling.h"
#elif defined(INTERRUPT)
  #include "dsp/dsp_interrupt.h"
#else
  #include "dsp/dsp_dma.h"
#endif

/******************************************************************************/
#define WORKSPACE_SIZE	128*1024
rtems_unsigned32 rtems_workspace_size = WORKSPACE_SIZE;
rtems_unsigned32 rtems_workspace_start[WORKSPACE_SIZE];
/******************************************************************************/

void Error(char *error)
{
	TRACE(error);
	exit(1);
}

static rtems_task panel_task(rtems_task_argument argument)
{
	static rtems_signed32	volume_table[128];
	rtems_unsigned8		potentiometer;
	rtems_unsigned8		value;
	int			panel, dsp;
	char			text[17];
	int			i;
	float			dB;

	TRACE("Starting panel_task...\n");

	/* Fill volume_table with a variable gain from -60 dB to 0 dB */
	for (i=0; i<128 ;i++)
	{
		if (i < 27)
			dB = -90.0 + (float) 40.0*i/27.0;
		else
			dB = -50.0 + (float) 50.0*(i-27)/100.0;

		volume_table[i] = float_to_fix_round(pow(10.0, dB/20.0));
	}

	volume_table[0] = 0;

	panel = panel_init();
	if (!panel)
		Error("ERROR: cannot access the Panel.\n");


	dsp = dsp_init(1, dspCode);
	if (!dsp)
		Error("ERROR: cannot access the DSP.\n");

#if defined(POLLING)
	panel_out_lcd_print(panel, 0, 0, "Poll audio thru ");
#elif defined(INTERRUPT)
	panel_out_lcd_print(panel, 0, 0, "Int. audio thru ");
#else
	panel_out_lcd_print(panel, 0, 0, " DMA audio thru ");
#endif

	while (TRUE)
	{
		if (!panel_in_new_event(panel, TRUE))
			Error("ERROR: unexpected exit waiting new panel event.\n");

		if (panel_in_potentiometer(panel, &potentiometer, &value))
		{
			switch (potentiometer)
			{
				case PANEL01_POT_VOLUME:
					if (!dsp_write_data(dsp, &volume_table[value], 1))
						Error("ERROR: cannot write to DSP.\n");

					sprintf(text, "   Volume %-3d   ", value);
					panel_out_lcd_print(panel, 1, 0, text);
					TRACE("Volume %-3d\n", value);
					break;
			}
		}
	}

	panel_exit(panel);

	TRACE("Ending panel_task...\n");
	rtems_task_delete(RTEMS_SELF);
}

rtems_boolean create_panel_task(void)
{
	rtems_id		task_id;
	rtems_status_code	status;

	status = rtems_task_create(
		rtems_build_name('T', 'H', 'R', 'U'),
		50,
		RTEMS_MINIMUM_STACK_SIZE,
		RTEMS_DEFAULT_MODES,
		RTEMS_DEFAULT_ATTRIBUTES,
		&task_id
	);

	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot create panel_task.\n");
		return FALSE;
	}

	status = rtems_task_start(task_id, panel_task, 0);
	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot start panel_task.\n");
		return FALSE;
	}

	return TRUE;
}

rtems_task rtems_main(rtems_task_argument ignored)
{
	TRACE("******************************************************************************\n");
#if defined(POLLING)
	TRACE("Polling audio thru example\n");
#elif defined(INTERRUPT)
	TRACE("Interrupt audio thru example\n");
#else
	TRACE("DMA audio thru example\n");
#endif
	TRACE("******************************************************************************\n");

	create_panel_task();
	rtems_task_delete(RTEMS_SELF);
}
